@@ -8,8 +8,6 @@ from typing import Any, Dict, List, Literal, Tuple, cast
 
 import pytest
 
-sys.path.insert(0, os.path.abspath(os.path.dirname(__file__) + "/../"))
-
 
 def ffmpeg_normalize_call(args: List[str]) -> Tuple[str, str]:
     cmd = [sys.executable, "-m", "ffmpeg_normalize"]
@@ -355,13 +353,23 @@ class TestFFmpegNormalize:
         assert os.path.isfile("normalized/test.mkv")
 
     def test_extra_output_options_json(self):
-        ffmpeg_normalize_call(["tests/test.mp4", "-c:a", "aac", "-e", '[ "-vbr", "3" ]'])
+        ffmpeg_normalize_call(
+            ["tests/test.mp4", "-c:a", "aac", "-e", '[ "-vbr", "3" ]']
+        )
         # FIXME: some better test that options are respected?
         assert os.path.isfile("normalized/test.mkv")
 
     def test_ofmt_fail(self):
         _, stderr = ffmpeg_normalize_call(
-            ["tests/test.mp4", "-ofmt", "mp3", "-o", "normalized/test.mp3", "-vn", "-sn"]
+            [
+                "tests/test.mp4",
+                "-ofmt",
+                "mp3",
+                "-o",
+                "normalized/test.mp3",
+                "-vn",
+                "-sn",
+            ]
         )
         assert "does not support" in stderr
 
@@ -400,13 +408,13 @@ class TestFFmpegNormalize:
         assert os.path.isfile("normalized/test.mkv")
 
     def test_duration(self):
-        _, stderr = ffmpeg_normalize_call(["tests/test.wav", "--debug"])
+        _, stderr = ffmpeg_normalize_call(["tests/test.m4a", "--debug"])
         assert "Found duration: " in stderr
 
     def test_pre_filters(self):
         ffmpeg_normalize_call(
             [
-                "tests/test.wav",
+                "tests/test.m4a",
                 "-o",
                 "normalized/test2.wav",
                 "-prf",
@@ -422,16 +430,16 @@ class TestFFmpegNormalize:
                     "output_file": "normalized/test2.mkv",
                     "stream_id": 0,
                     "ebu_pass1": {
-                        "input_i": -23.01,
-                        "input_tp": -10.75,
-                        "input_lra": 2.20,
-                        "input_thresh": -33.06,
-                        "output_i": -22.16,
-                        "output_tp": -9.46,
-                        "output_lra": 2.10,
-                        "output_thresh": -32.25,
+                        "input_i": -23.03,
+                        "input_tp": -17.86,
+                        "input_lra": 0.4,
+                        "input_thresh": -35.86,
+                        "output_i": -19.91,
+                        "output_tp": -14.76,
+                        "output_lra": 0.0,
+                        "output_thresh": -32.59,
                         "normalization_type": "dynamic",
-                        "target_offset": -0.84,
+                        "target_offset": -3.09,
                     },
                     "ebu_pass2": None,
                     "mean": None,
@@ -443,7 +451,7 @@ class TestFFmpegNormalize:
     def test_post_filters(self):
         ffmpeg_normalize_call(
             [
-                "tests/test.wav",
+                "tests/test.m4a",
                 "-o",
                 "normalized/test2.wav",
                 "-pof",
@@ -459,16 +467,16 @@ class TestFFmpegNormalize:
                     "output_file": "normalized/test2.mkv",
                     "stream_id": 0,
                     "ebu_pass1": {
-                        "input_i": -35.02,
-                        "input_tp": -22.76,
-                        "input_lra": 2.20,
-                        "input_thresh": -45.07,
-                        "output_i": -22.16,
-                        "output_tp": -9.46,
-                        "output_lra": 2.10,
-                        "output_thresh": -32.24,
+                        "input_i": -35.05,
+                        "input_tp": -29.9,
+                        "input_lra": 0.4,
+                        "input_thresh": -47.87,
+                        "output_i": -19.9,
+                        "output_tp": -14.76,
+                        "output_lra": 0.0,
+                        "output_thresh": -32.58,
                         "normalization_type": "dynamic",
-                        "target_offset": -0.84,
+                        "target_offset": -3.1,
                     },
                     "ebu_pass2": None,
                     "mean": None,
@@ -518,3 +526,76 @@ class TestFFmpegNormalize:
             # git checkout the files!
             for file in REPLAYGAIN_FILES:
                 subprocess.run(["git", "checkout", file], check=False)
+
+    def test_audio_streams_single(self):
+        """Test normalizing only a single audio stream"""
+        ffmpeg_normalize_call(["tests/test.mp4", "-as", "1", "-nt", "ebu"])
+        assert os.path.isfile("normalized/test.mkv")
+        # Check that output has only 1 audio stream
+        streams = _get_stream_info("normalized/test.mkv")
+        audio_streams = [s for s in streams if s["codec_type"] == "audio"]
+        assert len(audio_streams) == 1
+        # Verify stats show only one stream was normalized
+        stats = _get_stats("normalized/test.mkv", "ebu")
+        assert len(stats) == 1
+        assert stats[0]["stream_id"] == 1
+
+    def test_audio_streams_multiple(self):
+        """Test normalizing multiple specific audio streams"""
+        ffmpeg_normalize_call(["tests/test.mp4", "-as", "1,2", "-nt", "ebu"])
+        assert os.path.isfile("normalized/test.mkv")
+        # Check that output has 2 audio streams
+        streams = _get_stream_info("normalized/test.mkv")
+        audio_streams = [s for s in streams if s["codec_type"] == "audio"]
+        assert len(audio_streams) == 2
+        # Verify stats show both streams were normalized
+        stats = _get_stats("normalized/test.mkv", "ebu")
+        assert len(stats) == 2
+
+    def test_audio_streams_with_keep_other(self):
+        """Test normalizing one stream while keeping others as passthrough"""
+        ffmpeg_normalize_call(
+            ["tests/test.mp4", "-as", "1", "--keep-other-audio", "-nt", "ebu"]
+        )
+        assert os.path.isfile("normalized/test.mkv")
+        # Check that output has 2 audio streams (1 normalized, 1 passthrough)
+        streams = _get_stream_info("normalized/test.mkv")
+        audio_streams = [s for s in streams if s["codec_type"] == "audio"]
+        assert len(audio_streams) == 2
+        # First audio stream should be normalized (PCM or similar)
+        assert "pcm" in audio_streams[0]["codec_name"]
+        # Second audio stream should be copied (ac3)
+        assert audio_streams[1]["codec_name"] == "ac3"
+
+    def test_audio_default_only(self):
+        """Test normalizing only default audio streams"""
+        # Note: test.mp4 has both audio streams marked as default
+        ffmpeg_normalize_call(["tests/test.mp4", "--audio-default-only", "-nt", "ebu"])
+        assert os.path.isfile("normalized/test.mkv")
+        # Since both streams are default, both should be normalized
+        streams = _get_stream_info("normalized/test.mkv")
+        audio_streams = [s for s in streams if s["codec_type"] == "audio"]
+        assert len(audio_streams) == 2
+        # Verify stats show both streams were normalized
+        stats = _get_stats("normalized/test.mkv", "ebu")
+        assert len(stats) == 2
+
+    def test_audio_streams_invalid_option_combination(self):
+        """Test that using both --audio-streams and --audio-default-only fails"""
+        _, stderr = ffmpeg_normalize_call(
+            ["tests/test.mp4", "-as", "1", "--audio-default-only"]
+        )
+        assert "Cannot use both" in stderr
+
+    def test_keep_other_and_keep_original_conflict(self):
+        """Test that using both --keep-other-audio and --keep-original-audio fails"""
+        _, stderr = ffmpeg_normalize_call(
+            [
+                "tests/test.mp4",
+                "-as",
+                "1",
+                "--keep-other-audio",
+                "--keep-original-audio",
+            ]
+        )
+        assert "Cannot use both" in stderr
